# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import variant_pb2 as variant__pb2
from . import agent_info_pb2 as agent__info__pb2
from . import controller_info_pb2 as controller__info__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='com.rookout',
  syntax='proto3',
  serialized_pb=_b('\n\x0emessages.proto\x12\x0b\x63om.rookout\x1a\rvariant.proto\x1a\x10\x61gent_info.proto\x1a\x15\x63ontroller_info.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"D\n\x0fNewAgentMessage\x12\x31\n\nagent_info\x18\x01 \x01(\x0b\x32\x1d.com.rookout.AgentInformation\">\n\nStackFrame\x12\x10\n\x08\x66ilename\x18\x01 \x01(\t\x12\x0c\n\x04line\x18\x02 \x01(\r\x12\x10\n\x08\x66unction\x18\x03 \x01(\t\"5\n\nStackTrace\x12\'\n\x06\x66rames\x18\x01 \x03(\x0b\x32\x17.com.rookout.StackFrame\"~\n\tException\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\t\x12&\n\x08instance\x18\x03 \x01(\x0b\x32\x14.com.rookout.Variant\x12*\n\ttraceback\x18\x04 \x01(\x0b\x32\x17.com.rookout.StackTrace\"\xe2\x03\n\nLogMessage\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08\x61gent_id\x18\x02 \x01(\t\x12/\n\x05level\x18\x03 \x01(\x0e\x32 .com.rookout.LogMessage.LogLevel\x12\x10\n\x08\x66ilename\x18\x04 \x01(\t\x12\x0c\n\x04line\x18\x05 \x01(\r\x12\x0c\n\x04text\x18\x06 \x01(\t\x12\'\n\targuments\x18\x07 \x03(\x0b\x32\x14.com.rookout.Variant\x12)\n\texception\x18\x08 \x01(\x0b\x32\x16.com.rookout.Exception\x12\x1d\n\x11\x66ormatted_message\x18\t \x01(\tB\x02\x18\x01\x12\x32\n\x10legacy_arguments\x18\n \x01(\x0b\x32\x14.com.rookout.VariantB\x02\x18\x01\x12\x12\n\nclass_name\x18\x0b \x01(\t\x12\x13\n\x0bmethod_name\x18\x0c \x01(\t\x12\x15\n\rcontroller_id\x18\r \x01(\t\"M\n\x08LogLevel\x12\t\n\x05TRACE\x10\x00\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x08\n\x04INFO\x10\x02\x12\x0b\n\x07WARNING\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x12\t\n\x05\x46\x41TAL\x10\x05\"\x80\x01\n\x11RuleStatusMessage\x12\x10\n\x08\x61gent_id\x18\x01 \x01(\t\x12\x0f\n\x07rule_id\x18\x02 \x01(\t\x12\x0e\n\x06\x61\x63tive\x18\x03 \x01(\t\x12!\n\x05\x65rror\x18\x04 \x01(\x0b\x32\x12.com.rookout.Error\x12\x15\n\rcontroller_id\x18\x05 \x01(\t\"]\n\x10\x41ugReportMessage\x12\x10\n\x08\x61gent_id\x18\x01 \x01(\t\x12\x0e\n\x06\x61ug_id\x18\x02 \x01(\t\x12\'\n\targuments\x18\x03 \x01(\x0b\x32\x14.com.rookout.Variant\"e\n\x07UserMsg\x12\x31\n\naug_report\x18\x01 \x01(\x0b\x32\x1d.com.rookout.AugReportMessage\x12\x11\n\treport_id\x18\x02 \x01(\t\x12\x14\n\x0cworkspace_id\x18\x03 \x01(\t\"!\n\rAddAugCommand\x12\x10\n\x08\x61ug_json\x18\x01 \x01(\t\"\"\n\x12InitialAugsCommand\x12\x0c\n\x04\x61ugs\x18\x01 \x03(\t\"\"\n\x10RemoveAugCommand\x12\x0e\n\x06\x61ug_id\x18\x01 \x01(\t\"\x15\n\x13InitFinishedMessage\"\x12\n\x10\x43learAugsCommand\"\x1c\n\x0bPingMessage\x12\r\n\x05value\x18\x01 \x01(\x05\"\x1f\n\x0c\x45rrorMessage\x12\x0f\n\x07message\x18\x01 \x01(\t\"e\n\x0e\x43\x32\x43InitMessage\x12:\n\x0e\x63ontrollerInfo\x18\x01 \x01(\x0b\x32\".com.rookout.ControllerInformation\x12\x17\n\x0f\x63onnectedAgents\x18\x02 \x03(\t\" \n\nAgentsList\x12\x12\n\nagents_ids\x18\x01 \x03(\t\":\n\x13\x41gentIdHitCountPair\x12\x10\n\x08\x61gent_id\x18\x01 \x01(\t\x12\x11\n\thit_count\x18\x02 \x01(\r\"f\n\x15HitCountUpdateMessage\x12\x0e\n\x06\x61ug_id\x18\x01 \x01(\t\x12=\n\x13hitCountInformation\x18\x02 \x03(\x0b\x32 .com.rookout.AgentIdHitCountPairb\x06proto3')
  ,
  dependencies=[variant__pb2.DESCRIPTOR,agent__info__pb2.DESCRIPTOR,controller__info__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])



_LOGMESSAGE_LOGLEVEL = _descriptor.EnumDescriptor(
  name='LogLevel',
  full_name='com.rookout.LogMessage.LogLevel',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TRACE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEBUG', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='INFO', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WARNING', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ERROR', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FATAL', index=5, number=5,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=843,
  serialized_end=920,
)
_sym_db.RegisterEnumDescriptor(_LOGMESSAGE_LOGLEVEL)


_NEWAGENTMESSAGE = _descriptor.Descriptor(
  name='NewAgentMessage',
  full_name='com.rookout.NewAgentMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agent_info', full_name='com.rookout.NewAgentMessage.agent_info', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=188,
)


_STACKFRAME = _descriptor.Descriptor(
  name='StackFrame',
  full_name='com.rookout.StackFrame',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filename', full_name='com.rookout.StackFrame.filename', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='line', full_name='com.rookout.StackFrame.line', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='function', full_name='com.rookout.StackFrame.function', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=190,
  serialized_end=252,
)


_STACKTRACE = _descriptor.Descriptor(
  name='StackTrace',
  full_name='com.rookout.StackTrace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frames', full_name='com.rookout.StackTrace.frames', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=254,
  serialized_end=307,
)


_EXCEPTION = _descriptor.Descriptor(
  name='Exception',
  full_name='com.rookout.Exception',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='com.rookout.Exception.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='message', full_name='com.rookout.Exception.message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='instance', full_name='com.rookout.Exception.instance', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='traceback', full_name='com.rookout.Exception.traceback', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=309,
  serialized_end=435,
)


_LOGMESSAGE = _descriptor.Descriptor(
  name='LogMessage',
  full_name='com.rookout.LogMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='com.rookout.LogMessage.timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='agent_id', full_name='com.rookout.LogMessage.agent_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='level', full_name='com.rookout.LogMessage.level', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='filename', full_name='com.rookout.LogMessage.filename', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='line', full_name='com.rookout.LogMessage.line', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='text', full_name='com.rookout.LogMessage.text', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='com.rookout.LogMessage.arguments', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='exception', full_name='com.rookout.LogMessage.exception', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='formatted_message', full_name='com.rookout.LogMessage.formatted_message', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='legacy_arguments', full_name='com.rookout.LogMessage.legacy_arguments', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=_descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))),
    _descriptor.FieldDescriptor(
      name='class_name', full_name='com.rookout.LogMessage.class_name', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='method_name', full_name='com.rookout.LogMessage.method_name', index=11,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='controller_id', full_name='com.rookout.LogMessage.controller_id', index=12,
      number=13, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LOGMESSAGE_LOGLEVEL,
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=438,
  serialized_end=920,
)


_RULESTATUSMESSAGE = _descriptor.Descriptor(
  name='RuleStatusMessage',
  full_name='com.rookout.RuleStatusMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agent_id', full_name='com.rookout.RuleStatusMessage.agent_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rule_id', full_name='com.rookout.RuleStatusMessage.rule_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='active', full_name='com.rookout.RuleStatusMessage.active', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='error', full_name='com.rookout.RuleStatusMessage.error', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='controller_id', full_name='com.rookout.RuleStatusMessage.controller_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=923,
  serialized_end=1051,
)


_AUGREPORTMESSAGE = _descriptor.Descriptor(
  name='AugReportMessage',
  full_name='com.rookout.AugReportMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agent_id', full_name='com.rookout.AugReportMessage.agent_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aug_id', full_name='com.rookout.AugReportMessage.aug_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='arguments', full_name='com.rookout.AugReportMessage.arguments', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1053,
  serialized_end=1146,
)


_USERMSG = _descriptor.Descriptor(
  name='UserMsg',
  full_name='com.rookout.UserMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='aug_report', full_name='com.rookout.UserMsg.aug_report', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='report_id', full_name='com.rookout.UserMsg.report_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='workspace_id', full_name='com.rookout.UserMsg.workspace_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1148,
  serialized_end=1249,
)


_ADDAUGCOMMAND = _descriptor.Descriptor(
  name='AddAugCommand',
  full_name='com.rookout.AddAugCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='aug_json', full_name='com.rookout.AddAugCommand.aug_json', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1251,
  serialized_end=1284,
)


_INITIALAUGSCOMMAND = _descriptor.Descriptor(
  name='InitialAugsCommand',
  full_name='com.rookout.InitialAugsCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='augs', full_name='com.rookout.InitialAugsCommand.augs', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1286,
  serialized_end=1320,
)


_REMOVEAUGCOMMAND = _descriptor.Descriptor(
  name='RemoveAugCommand',
  full_name='com.rookout.RemoveAugCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='aug_id', full_name='com.rookout.RemoveAugCommand.aug_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1322,
  serialized_end=1356,
)


_INITFINISHEDMESSAGE = _descriptor.Descriptor(
  name='InitFinishedMessage',
  full_name='com.rookout.InitFinishedMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1358,
  serialized_end=1379,
)


_CLEARAUGSCOMMAND = _descriptor.Descriptor(
  name='ClearAugsCommand',
  full_name='com.rookout.ClearAugsCommand',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1381,
  serialized_end=1399,
)


_PINGMESSAGE = _descriptor.Descriptor(
  name='PingMessage',
  full_name='com.rookout.PingMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='com.rookout.PingMessage.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1401,
  serialized_end=1429,
)


_ERRORMESSAGE = _descriptor.Descriptor(
  name='ErrorMessage',
  full_name='com.rookout.ErrorMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='message', full_name='com.rookout.ErrorMessage.message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1431,
  serialized_end=1462,
)


_C2CINITMESSAGE = _descriptor.Descriptor(
  name='C2CInitMessage',
  full_name='com.rookout.C2CInitMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='controllerInfo', full_name='com.rookout.C2CInitMessage.controllerInfo', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='connectedAgents', full_name='com.rookout.C2CInitMessage.connectedAgents', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1464,
  serialized_end=1565,
)


_AGENTSLIST = _descriptor.Descriptor(
  name='AgentsList',
  full_name='com.rookout.AgentsList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agents_ids', full_name='com.rookout.AgentsList.agents_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1567,
  serialized_end=1599,
)


_AGENTIDHITCOUNTPAIR = _descriptor.Descriptor(
  name='AgentIdHitCountPair',
  full_name='com.rookout.AgentIdHitCountPair',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='agent_id', full_name='com.rookout.AgentIdHitCountPair.agent_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hit_count', full_name='com.rookout.AgentIdHitCountPair.hit_count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1601,
  serialized_end=1659,
)


_HITCOUNTUPDATEMESSAGE = _descriptor.Descriptor(
  name='HitCountUpdateMessage',
  full_name='com.rookout.HitCountUpdateMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='aug_id', full_name='com.rookout.HitCountUpdateMessage.aug_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hitCountInformation', full_name='com.rookout.HitCountUpdateMessage.hitCountInformation', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1661,
  serialized_end=1763,
)

_NEWAGENTMESSAGE.fields_by_name['agent_info'].message_type = agent__info__pb2._AGENTINFORMATION
_STACKTRACE.fields_by_name['frames'].message_type = _STACKFRAME
_EXCEPTION.fields_by_name['instance'].message_type = variant__pb2._VARIANT
_EXCEPTION.fields_by_name['traceback'].message_type = _STACKTRACE
_LOGMESSAGE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LOGMESSAGE.fields_by_name['level'].enum_type = _LOGMESSAGE_LOGLEVEL
_LOGMESSAGE.fields_by_name['arguments'].message_type = variant__pb2._VARIANT
_LOGMESSAGE.fields_by_name['exception'].message_type = _EXCEPTION
_LOGMESSAGE.fields_by_name['legacy_arguments'].message_type = variant__pb2._VARIANT
_LOGMESSAGE_LOGLEVEL.containing_type = _LOGMESSAGE
_RULESTATUSMESSAGE.fields_by_name['error'].message_type = variant__pb2._ERROR
_AUGREPORTMESSAGE.fields_by_name['arguments'].message_type = variant__pb2._VARIANT
_USERMSG.fields_by_name['aug_report'].message_type = _AUGREPORTMESSAGE
_C2CINITMESSAGE.fields_by_name['controllerInfo'].message_type = controller__info__pb2._CONTROLLERINFORMATION
_HITCOUNTUPDATEMESSAGE.fields_by_name['hitCountInformation'].message_type = _AGENTIDHITCOUNTPAIR
DESCRIPTOR.message_types_by_name['NewAgentMessage'] = _NEWAGENTMESSAGE
DESCRIPTOR.message_types_by_name['StackFrame'] = _STACKFRAME
DESCRIPTOR.message_types_by_name['StackTrace'] = _STACKTRACE
DESCRIPTOR.message_types_by_name['Exception'] = _EXCEPTION
DESCRIPTOR.message_types_by_name['LogMessage'] = _LOGMESSAGE
DESCRIPTOR.message_types_by_name['RuleStatusMessage'] = _RULESTATUSMESSAGE
DESCRIPTOR.message_types_by_name['AugReportMessage'] = _AUGREPORTMESSAGE
DESCRIPTOR.message_types_by_name['UserMsg'] = _USERMSG
DESCRIPTOR.message_types_by_name['AddAugCommand'] = _ADDAUGCOMMAND
DESCRIPTOR.message_types_by_name['InitialAugsCommand'] = _INITIALAUGSCOMMAND
DESCRIPTOR.message_types_by_name['RemoveAugCommand'] = _REMOVEAUGCOMMAND
DESCRIPTOR.message_types_by_name['InitFinishedMessage'] = _INITFINISHEDMESSAGE
DESCRIPTOR.message_types_by_name['ClearAugsCommand'] = _CLEARAUGSCOMMAND
DESCRIPTOR.message_types_by_name['PingMessage'] = _PINGMESSAGE
DESCRIPTOR.message_types_by_name['ErrorMessage'] = _ERRORMESSAGE
DESCRIPTOR.message_types_by_name['C2CInitMessage'] = _C2CINITMESSAGE
DESCRIPTOR.message_types_by_name['AgentsList'] = _AGENTSLIST
DESCRIPTOR.message_types_by_name['AgentIdHitCountPair'] = _AGENTIDHITCOUNTPAIR
DESCRIPTOR.message_types_by_name['HitCountUpdateMessage'] = _HITCOUNTUPDATEMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

NewAgentMessage = _reflection.GeneratedProtocolMessageType('NewAgentMessage', (_message.Message,), dict(
  DESCRIPTOR = _NEWAGENTMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.NewAgentMessage)
  ))
_sym_db.RegisterMessage(NewAgentMessage)

StackFrame = _reflection.GeneratedProtocolMessageType('StackFrame', (_message.Message,), dict(
  DESCRIPTOR = _STACKFRAME,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.StackFrame)
  ))
_sym_db.RegisterMessage(StackFrame)

StackTrace = _reflection.GeneratedProtocolMessageType('StackTrace', (_message.Message,), dict(
  DESCRIPTOR = _STACKTRACE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.StackTrace)
  ))
_sym_db.RegisterMessage(StackTrace)

Exception = _reflection.GeneratedProtocolMessageType('Exception', (_message.Message,), dict(
  DESCRIPTOR = _EXCEPTION,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.Exception)
  ))
_sym_db.RegisterMessage(Exception)

LogMessage = _reflection.GeneratedProtocolMessageType('LogMessage', (_message.Message,), dict(
  DESCRIPTOR = _LOGMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.LogMessage)
  ))
_sym_db.RegisterMessage(LogMessage)

RuleStatusMessage = _reflection.GeneratedProtocolMessageType('RuleStatusMessage', (_message.Message,), dict(
  DESCRIPTOR = _RULESTATUSMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.RuleStatusMessage)
  ))
_sym_db.RegisterMessage(RuleStatusMessage)

AugReportMessage = _reflection.GeneratedProtocolMessageType('AugReportMessage', (_message.Message,), dict(
  DESCRIPTOR = _AUGREPORTMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.AugReportMessage)
  ))
_sym_db.RegisterMessage(AugReportMessage)

UserMsg = _reflection.GeneratedProtocolMessageType('UserMsg', (_message.Message,), dict(
  DESCRIPTOR = _USERMSG,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.UserMsg)
  ))
_sym_db.RegisterMessage(UserMsg)

AddAugCommand = _reflection.GeneratedProtocolMessageType('AddAugCommand', (_message.Message,), dict(
  DESCRIPTOR = _ADDAUGCOMMAND,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.AddAugCommand)
  ))
_sym_db.RegisterMessage(AddAugCommand)

InitialAugsCommand = _reflection.GeneratedProtocolMessageType('InitialAugsCommand', (_message.Message,), dict(
  DESCRIPTOR = _INITIALAUGSCOMMAND,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.InitialAugsCommand)
  ))
_sym_db.RegisterMessage(InitialAugsCommand)

RemoveAugCommand = _reflection.GeneratedProtocolMessageType('RemoveAugCommand', (_message.Message,), dict(
  DESCRIPTOR = _REMOVEAUGCOMMAND,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.RemoveAugCommand)
  ))
_sym_db.RegisterMessage(RemoveAugCommand)

InitFinishedMessage = _reflection.GeneratedProtocolMessageType('InitFinishedMessage', (_message.Message,), dict(
  DESCRIPTOR = _INITFINISHEDMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.InitFinishedMessage)
  ))
_sym_db.RegisterMessage(InitFinishedMessage)

ClearAugsCommand = _reflection.GeneratedProtocolMessageType('ClearAugsCommand', (_message.Message,), dict(
  DESCRIPTOR = _CLEARAUGSCOMMAND,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.ClearAugsCommand)
  ))
_sym_db.RegisterMessage(ClearAugsCommand)

PingMessage = _reflection.GeneratedProtocolMessageType('PingMessage', (_message.Message,), dict(
  DESCRIPTOR = _PINGMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.PingMessage)
  ))
_sym_db.RegisterMessage(PingMessage)

ErrorMessage = _reflection.GeneratedProtocolMessageType('ErrorMessage', (_message.Message,), dict(
  DESCRIPTOR = _ERRORMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.ErrorMessage)
  ))
_sym_db.RegisterMessage(ErrorMessage)

C2CInitMessage = _reflection.GeneratedProtocolMessageType('C2CInitMessage', (_message.Message,), dict(
  DESCRIPTOR = _C2CINITMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.C2CInitMessage)
  ))
_sym_db.RegisterMessage(C2CInitMessage)

AgentsList = _reflection.GeneratedProtocolMessageType('AgentsList', (_message.Message,), dict(
  DESCRIPTOR = _AGENTSLIST,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.AgentsList)
  ))
_sym_db.RegisterMessage(AgentsList)

AgentIdHitCountPair = _reflection.GeneratedProtocolMessageType('AgentIdHitCountPair', (_message.Message,), dict(
  DESCRIPTOR = _AGENTIDHITCOUNTPAIR,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.AgentIdHitCountPair)
  ))
_sym_db.RegisterMessage(AgentIdHitCountPair)

HitCountUpdateMessage = _reflection.GeneratedProtocolMessageType('HitCountUpdateMessage', (_message.Message,), dict(
  DESCRIPTOR = _HITCOUNTUPDATEMESSAGE,
  __module__ = 'messages_pb2'
  # @@protoc_insertion_point(class_scope:com.rookout.HitCountUpdateMessage)
  ))
_sym_db.RegisterMessage(HitCountUpdateMessage)


_LOGMESSAGE.fields_by_name['formatted_message'].has_options = True
_LOGMESSAGE.fields_by_name['formatted_message']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
_LOGMESSAGE.fields_by_name['legacy_arguments'].has_options = True
_LOGMESSAGE.fields_by_name['legacy_arguments']._options = _descriptor._ParseOptions(descriptor_pb2.FieldOptions(), _b('\030\001'))
# @@protoc_insertion_point(module_scope)
